{
    "$graph": [
        {
            "class": "CommandLineTool",
            "id": "#Dockstore.cwl",
            "label": "Seqware-BWA-Workflow",
            "requirements": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "quay.io/pancancer/pcawg-bwa-mem-workflow:2.6.8_1.2"
                },
                {
                    "class": "InlineJavascriptRequirement"
                }
            ],
            "inputs": [
                {
                    "type": "string",
                    "inputBinding": {
                        "position": 11,
                        "prefix": "--download-reference-files"
                    },
                    "doc": "should we attempt to download the reference files [\"true\", \"false\"]",
                    "id": "#Dockstore.cwl/Seqware-BWA-Workflow/download_reference_files"
                },
                {
                    "type": "string",
                    "inputBinding": {
                        "position": 9,
                        "prefix": "--output-dir"
                    },
                    "doc": "the output directory",
                    "id": "#Dockstore.cwl/Seqware-BWA-Workflow/output_dir"
                },
                {
                    "type": "string",
                    "inputBinding": {
                        "position": 10,
                        "prefix": "--output-file-basename"
                    },
                    "doc": "the basename to use for output files",
                    "id": "#Dockstore.cwl/Seqware-BWA-Workflow/output_file_basename"
                },
                {
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--files"
                    },
                    "id": "#Dockstore.cwl/Seqware-BWA-Workflow/reads"
                },
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 2,
                        "prefix": "--reference-gz"
                    },
                    "doc": "the reference *.fa.gz file",
                    "id": "#Dockstore.cwl/Seqware-BWA-Workflow/reference_gz"
                },
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 4,
                        "prefix": "--reference-gz-amb"
                    },
                    "doc": "the reference *.fa.gz.amb file",
                    "id": "#Dockstore.cwl/Seqware-BWA-Workflow/reference_gz_amb"
                },
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 5,
                        "prefix": "--reference-gz-ann"
                    },
                    "doc": "the reference *.fa.gz.ann file",
                    "id": "#Dockstore.cwl/Seqware-BWA-Workflow/reference_gz_ann"
                },
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 6,
                        "prefix": "--reference-gz-bwt"
                    },
                    "doc": "the reference *.fa.gz.bwt file",
                    "id": "#Dockstore.cwl/Seqware-BWA-Workflow/reference_gz_bwt"
                },
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 3,
                        "prefix": "--reference-gz-fai"
                    },
                    "doc": "the reference *.fa.gz.fai file",
                    "id": "#Dockstore.cwl/Seqware-BWA-Workflow/reference_gz_fai"
                },
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 7,
                        "prefix": "--reference-gz-pac"
                    },
                    "doc": "the reference *.fa.gz.pac file",
                    "id": "#Dockstore.cwl/Seqware-BWA-Workflow/reference_gz_pac"
                },
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 8,
                        "prefix": "--reference-gz-sa"
                    },
                    "doc": "the reference *.fa.gz.sa file",
                    "id": "#Dockstore.cwl/Seqware-BWA-Workflow/reference_gz_sa"
                }
            ],
            "baseCommand": [
                "/start.sh",
                "python",
                "/home/seqware/Seqware-BWA-Workflow/run_seqware_workflow.py"
            ],
            "doc": "\nPCAWG uniform alignment workflow uses the popular short read aligner tool BWA MEM (https://github.com/lh3/bwa)\nwith BioBAMBAM (https://github.com/gt1/biobambam) for BAM sorting, merging and marking duplicate.\nThe alignment workflow has been dockerized and packaged using CWL workflow language, the source code\nis available on GitHub at: https://github.com/ICGC-TCGA-PanCancer/Seqware-BWA-Workflow.\n\n## Run the workflow with your own data\n### Prepare compute environment and install software packages\nThe workflow has been tested in Ubuntu 16.04 Linux environment with the following hardware\nand software settings.\n\n#### Hardware requirement (assuming 30X coverage whole genome sequence)\n- CPU core: 16\n- Memory: 64GB\n- Disk space: 1TB\n\n#### Software installation\n- Docker (1.12.6): follow instructions to install Docker https://docs.docker.com/engine/installation\n- CWL tool\n```\npip install cwltool==1.0.20170217172322\n```\n\n### Prepare input data\n#### Input unaligned BAM files\n\nThe workflow uses lane-level unaligned BAM files as input, one BAM per lane (aka read group).\nPlease ensure *@RG* field is populated properly in the BAM header, the following is a\nvalid *@RG* entry. *ID* field has to be unique among your dataset.\n```\n@RG\tID:WTSI:9399_7\tCN:WTSI\tPL:ILLUMINA\tPM:Illumina HiSeq 2000\tLB:WGS:WTSI:28085\tPI:453\tSM:f393ba16-9361-5df4-e040-11ac0d4844e8\tPU:WTSI:9399_7\tDT:2013-03-18T00:00:00+00:00\n```\nMultiple unaligned BAMs from the same sample (with same *SM* value) should be run together. *SM* is\nglobally unique UUID for the sample. Put the input BAM files in a subfolder. In this example,\nwe have two BAMs in a folder named *bams*.\n\n\n#### Reference genome sequence files\n\nThe reference genome files can be downloaded from the ICGC Data Portal at\nunder https://dcc.icgc.org/releases/PCAWG/reference_data/pcawg-bwa-mem. Please download all\nreference files and put them under a subfolder called *reference*.\n\n#### Job JSON file for CWL\n\nFinally, we need to prepare a JSON file with input, reference and output files specified. Please\nreplace the *reads* parameter with your real BAM file name.\n\nName the JSON file: *pcawg-bwa-mem-aligner.job.json*\n```\n{\n  \"reads\": [\n    {\n      \"path\":\"bams/seq_from_normal_sample_A.lane_1.bam\",\n      \"class\":\"File\"\n    },\n    {\n      \"path\":\"bams/seq_from_normal_sample_A.lane_2.bam\",\n      \"class\":\"File\"\n    }\n  ],\n  \"output_dir\": \"datastore\",\n  \"output_file_basename\": \"seq_from_normal_sample_A\",\n  \"reference_gz_amb\": {\n    \"path\": \"reference/genome.fa.gz.64.amb\",\n    \"class\": \"File\"\n  },\n  \"reference_gz_sa\": {\n    \"path\": \"reference/genome.fa.gz.64.sa\",\n    \"class\": \"File\"\n  },\n  \"reference_gz_pac\": {\n    \"path\": \"reference/genome.fa.gz.64.pac\",\n    \"class\": \"File\"\n  },\n  \"reference_gz_ann\": {\n    \"path\": \"reference/genome.fa.gz.64.ann\",\n    \"class\": \"File\"\n  },\n  \"reference_gz_bwt\": {\n    \"path\": \"reference/genome.fa.gz.64.bwt\",\n    \"class\": \"File\"\n  },\n  \"reference_gz_fai\": {\n    \"path\": \"reference/genome.fa.gz.fai\",\n    \"class\": \"File\"\n  },\n  \"reference_gz\": {\n    \"path\": \"reference/genome.fa.gz\",\n    \"class\": \"File\"\n  }\n}\n```\n\n### Run the workflow\n#### Option 1: Run with CWL tool\n- Download CWL workflow definition file\n```\nwget -O pcawg-bwa-mem-aligner.cwl \"https://raw.githubusercontent.com/ICGC-TCGA-PanCancer/Seqware-BWA-Workflow/2.6.8_1.3/Dockstore.cwl\"\n```\n\n- Run *cwltool* to execute the workflow\n```\nnohup cwltool --debug --non-strict pcawg-bwa-mem-aligner.cwl pcawg-bwa-mem-aligner.job.json > pcawg-bwa-mem-aligner.log 2>&1 &\n```\n\n#### Option 2: Run with the Dockstore CLI\nSee the *Launch with* section below for details\n",
            "http://purl.org/dc/terms/creator": {
                "@id": "http://orcid.org/0000-0002-7681-6415",
                "foaf:name": "Brian O'Connor",
                "foaf:mbox": "mailto:briandoconnor@gmail.com"
            },
            "http://purl.org/dc/terms/contributor": {
                "foaf:name": "Denis Yuen",
                "foaf:mbox": "mailto:denis.yuen@oicr.on.ca"
            },
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "$(inputs.output_dir + '/' + inputs.output_file_basename + '.bam.bai')"
                    },
                    "id": "#Dockstore.cwl/Seqware-BWA-Workflow/merged_output_bai"
                },
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "$(inputs.output_dir + '/' + inputs.output_file_basename + '.bam')"
                    },
                    "id": "#Dockstore.cwl/Seqware-BWA-Workflow/merged_output_bam"
                },
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "$(inputs.output_dir + '/' + inputs.output_file_basename + '.bam.metrics')"
                    },
                    "id": "#Dockstore.cwl/Seqware-BWA-Workflow/merged_output_metrics"
                },
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "$(inputs.output_dir + '/' + inputs.output_file_basename + '.unmapped.bam.bai')"
                    },
                    "id": "#Dockstore.cwl/Seqware-BWA-Workflow/merged_output_unmapped_bai"
                },
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "$(inputs.output_dir + '/' + inputs.output_file_basename + '.unmapped.bam')"
                    },
                    "id": "#Dockstore.cwl/Seqware-BWA-Workflow/merged_output_unmapped_bam"
                },
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "$(inputs.output_dir + '/' + inputs.output_file_basename + '.unmapped.bam.metrics')"
                    },
                    "id": "#Dockstore.cwl/Seqware-BWA-Workflow/merged_output_unmapped_metrics"
                }
            ]
        },
        {
            "class": "Workflow",
            "requirements": [
                {
                    "class": "MultipleInputFeatureRequirement"
                }
            ],
            "inputs": [
                {
                    "type": "string",
                    "inputBinding": {
                        "position": 11,
                        "prefix": "--download-reference-files"
                    },
                    "doc": "should we attempt to download the reference files [\"true\", \"false\"]",
                    "id": "#main/download_reference_files"
                },
                {
                    "type": "string",
                    "inputBinding": {
                        "position": 9,
                        "prefix": "--output-dir"
                    },
                    "doc": "the output directory",
                    "id": "#main/output_dir"
                },
                {
                    "type": "string",
                    "inputBinding": {
                        "position": 10,
                        "prefix": "--output-file-basename"
                    },
                    "doc": "the basename to use for output files",
                    "id": "#main/output_file_basename"
                },
                {
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--files"
                    },
                    "id": "#main/reads"
                },
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 2,
                        "prefix": "--reference-gz"
                    },
                    "doc": "the reference *.fa.gz file",
                    "id": "#main/reference_gz"
                },
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 4,
                        "prefix": "--reference-gz-amb"
                    },
                    "doc": "the reference *.fa.gz.amb file",
                    "id": "#main/reference_gz_amb"
                },
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 5,
                        "prefix": "--reference-gz-ann"
                    },
                    "doc": "the reference *.fa.gz.ann file",
                    "id": "#main/reference_gz_ann"
                },
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 6,
                        "prefix": "--reference-gz-bwt"
                    },
                    "doc": "the reference *.fa.gz.bwt file",
                    "id": "#main/reference_gz_bwt"
                },
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 3,
                        "prefix": "--reference-gz-fai"
                    },
                    "doc": "the reference *.fa.gz.fai file",
                    "id": "#main/reference_gz_fai"
                },
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 7,
                        "prefix": "--reference-gz-pac"
                    },
                    "doc": "the reference *.fa.gz.pac file",
                    "id": "#main/reference_gz_pac"
                },
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 8,
                        "prefix": "--reference-gz-sa"
                    },
                    "doc": "the reference *.fa.gz.sa file",
                    "id": "#main/reference_gz_sa"
                },
                {
                    "type": "File",
                    "id": "#main/reference_result_normal_bai"
                },
                {
                    "type": "File",
                    "id": "#main/reference_result_normal_bam"
                },
                {
                    "type": "File",
                    "id": "#main/reference_result_normal_metrics"
                },
                {
                    "type": "File",
                    "id": "#main/reference_result_normal_unmapped_bai"
                },
                {
                    "type": "File",
                    "id": "#main/reference_result_normal_unmapped_bam"
                },
                {
                    "type": "File",
                    "id": "#main/reference_result_normal_unmapped_metrics"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputSource": "#main/checker/stderr_log",
                    "id": "#main/workflow_err_file"
                },
                {
                    "type": "File",
                    "outputSource": "#main/checker/stdout_log",
                    "id": "#main/workflow_output_file"
                }
            ],
            "steps": [
                {
                    "run": "#pcawg-bwa-mem-result-checker.cwl",
                    "in": [
                        {
                            "source": [
                                "#main/pcawg_bwa_mem_aligner/merged_output_bam",
                                "#main/reference_result_normal_bam",
                                "#main/pcawg_bwa_mem_aligner/merged_output_unmapped_bam",
                                "#main/reference_result_normal_unmapped_bam",
                                "#main/pcawg_bwa_mem_aligner/merged_output_bai",
                                "#main/reference_result_normal_bai",
                                "#main/pcawg_bwa_mem_aligner/merged_output_unmapped_bai",
                                "#main/reference_result_normal_unmapped_bai",
                                "#main/pcawg_bwa_mem_aligner/merged_output_metrics",
                                "#main/reference_result_normal_metrics",
                                "#main/pcawg_bwa_mem_aligner/merged_output_unmapped_metrics",
                                "#main/reference_result_normal_unmapped_metrics"
                            ],
                            "id": "#main/checker/result_files"
                        }
                    ],
                    "out": [
                        "#main/checker/stdout_log",
                        "#main/checker/stderr_log"
                    ],
                    "id": "#main/checker"
                },
                {
                    "run": "#Dockstore.cwl",
                    "in": [
                        {
                            "source": "#main/download_reference_files",
                            "id": "#main/pcawg_bwa_mem_aligner/download_reference_files"
                        },
                        {
                            "source": "#main/output_dir",
                            "id": "#main/pcawg_bwa_mem_aligner/output_dir"
                        },
                        {
                            "source": "#main/output_file_basename",
                            "id": "#main/pcawg_bwa_mem_aligner/output_file_basename"
                        },
                        {
                            "source": "#main/reads",
                            "id": "#main/pcawg_bwa_mem_aligner/reads"
                        },
                        {
                            "source": "#main/reference_gz",
                            "id": "#main/pcawg_bwa_mem_aligner/reference_gz"
                        },
                        {
                            "source": "#main/reference_gz_amb",
                            "id": "#main/pcawg_bwa_mem_aligner/reference_gz_amb"
                        },
                        {
                            "source": "#main/reference_gz_ann",
                            "id": "#main/pcawg_bwa_mem_aligner/reference_gz_ann"
                        },
                        {
                            "source": "#main/reference_gz_bwt",
                            "id": "#main/pcawg_bwa_mem_aligner/reference_gz_bwt"
                        },
                        {
                            "source": "#main/reference_gz_fai",
                            "id": "#main/pcawg_bwa_mem_aligner/reference_gz_fai"
                        },
                        {
                            "source": "#main/reference_gz_pac",
                            "id": "#main/pcawg_bwa_mem_aligner/reference_gz_pac"
                        },
                        {
                            "source": "#main/reference_gz_sa",
                            "id": "#main/pcawg_bwa_mem_aligner/reference_gz_sa"
                        }
                    ],
                    "out": [
                        "#main/pcawg_bwa_mem_aligner/merged_output_bai",
                        "#main/pcawg_bwa_mem_aligner/merged_output_unmapped_metrics",
                        "#main/pcawg_bwa_mem_aligner/merged_output_bam",
                        "#main/pcawg_bwa_mem_aligner/merged_output_metrics",
                        "#main/pcawg_bwa_mem_aligner/merged_output_unmapped_bai",
                        "#main/pcawg_bwa_mem_aligner/merged_output_unmapped_bam"
                    ],
                    "id": "#main/pcawg_bwa_mem_aligner"
                }
            ],
            "id": "#main"
        },
        {
            "class": "CommandLineTool",
            "hints": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "quay.io/baminou/pcawg-bwa-mem-aligner-ga4gh-result-checker:2.0"
                }
            ],
            "inputs": [
                {
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "inputBinding": {
                        "position": 1
                    },
                    "id": "#pcawg-bwa-mem-result-checker.cwl/result_files"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "log.stderr"
                    },
                    "doc": "File containing error result",
                    "id": "#pcawg-bwa-mem-result-checker.cwl/stderr_log"
                },
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "log.stdout"
                    },
                    "doc": "File containing result",
                    "id": "#pcawg-bwa-mem-result-checker.cwl/stdout_log"
                }
            ],
            "baseCommand": [
                "bash",
                "/usr/local/bin/pcawg-bwa-mem-result-checker.sh"
            ],
            "id": "#pcawg-bwa-mem-result-checker.cwl"
        }
    ],
    "cwlVersion": "v1.0"
}
